﻿using System;
using System.Drawing;
using System.Windows.Forms;
using ThinkGeo.MapSuite.DesktopEdition;
using ThinkGeo.MapSuite.Core;
using System.Collections.ObjectModel;

namespace TestGenerateTilingImages
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        private void btnGenerate_Click(object sender, EventArgs e)
        {
            Collection<ZoomLevel> zoomLevels = GetAllZoomLevels();
            int tileWidth = 512;
            int tileHeight = 512;

            EcwRasterLayer ecwImageLayer = new EcwRasterLayer(@"C:\Program Files\ThinkGeo\Map Suite Desktop Full Edition 3.0\Samples\SampleData\Data\World.ecw");
            ecwImageLayer.UpperThreshold = double.MaxValue;
            ecwImageLayer.LowerThreshold = 0;

            MapEngine mapEngine = new MapEngine();
            mapEngine.StaticLayers.Add(ecwImageLayer);
            GeographyUnit mapUnit = GeographyUnit.DecimalDegree;

            ecwImageLayer.Open();
            RectangleShape cacheExtent = ecwImageLayer.GetBoundingBox();
            ecwImageLayer.Close();

            for (int i = 0; i < zoomLevels.Count; i++)
            {
                MapSuiteTileMatrix tileMatrix = new MapSuiteTileMatrix(zoomLevels[i].Scale, tileWidth, tileHeight, mapUnit);
                BitmapTileCache tileCache = new FileBitmapTileCache(txtCachePath.Text, string.Empty, TileImageFormat.Png, tileMatrix);
                RowColumnRange rowColumnRange = tileCache.TileMatrix.GetIntersectingRowColumnRange(cacheExtent);

                Bitmap bitmap = null;
                RectangleShape tileExtent = tileCache.TileMatrix.GetCell(0, 0).BoundingBox;
                    
                if (tileExtent.Width > cacheExtent.Width || tileExtent.Height > cacheExtent.Height)
                {
                    mapEngine.CurrentExtent = tileExtent;
                    bitmap = new Bitmap(tileWidth, tileHeight);
                }
                else
                {
                    int bitMapWidth = tileWidth * ((int)(rowColumnRange.MaxColumnIndex - rowColumnRange.MinColumnIndex) + 1);
                    int bitMapHeight = tileHeight * ((int)(rowColumnRange.MaxRowIndex - rowColumnRange.MinRowIndex) + 1);
                    mapEngine.CurrentExtent = cacheExtent;
                    // if the bitmapWidth is very big, you need to optimize the way generating tiles, otherwise the bitmap will be too big for the memory.
                    // For example, you can always generate a 1024 * 1024 bitmap and then split it into 4 512*512 tiles
                    bitmap = new Bitmap(bitMapWidth, bitMapHeight);
                }

                mapEngine.OpenAllLayers();
                mapEngine.DrawStaticLayers(bitmap, mapUnit);
                mapEngine.DrawDynamicLayers(bitmap, mapUnit);
                mapEngine.CloseAllLayers();

                tileCache.SaveTiles((Bitmap)bitmap.Clone(), mapEngine.CurrentExtent);
                bitmap.Dispose();
            }
        }

        private void btnLoadMap_Click(object sender, EventArgs e)
        {
            string versiong = WinformsMap.GetVersion();
            winformsMap1.MapUnit = GeographyUnit.DecimalDegree;
            winformsMap1.CurrentExtent = new RectangleShape(-118.098, 84.3, 118.098, -84.3);
            winformsMap1.BackgroundOverlay.BackgroundBrush = new GeoSolidBrush(GeoColor.GeographicColors.ShallowOcean);

            EcwRasterLayer ecwImageLayer = new EcwRasterLayer(@"C:\Program Files\ThinkGeo\Map Suite Desktop Full Edition 3.0\Samples\SampleData\Data\World.ecw");
            ecwImageLayer.UpperThreshold = double.MaxValue;
            ecwImageLayer.LowerThreshold = 0;

            FileBitmapTileCache fileBitmapTileCache = new FileBitmapTileCache(txtCachePath.Text, string.Empty);
            fileBitmapTileCache.TileMatrix.TileHeight = 512;
            fileBitmapTileCache.TileMatrix.TileWidth = 512;

            LayerOverlay ImageOverlay = new LayerOverlay();
            ImageOverlay.Layers.Add("EcwImageLayer", ecwImageLayer);
            ImageOverlay.TileCache = fileBitmapTileCache;

            winformsMap1.Overlays.Add(ImageOverlay);

            winformsMap1.Refresh();
        }


        private Collection<ZoomLevel> GetAllZoomLevels()
        {
            // We only add the top 5 ZoomLevels here for example. 
            // If you want to add more scales, you need to optimize the way generating tiles otherwise 
            // the bitmap we generated might be too big to be hold in memory.
            ZoomLevelSet zoomLevelSet = new ZoomLevelSet();
            Collection<ZoomLevel> resultZoomLevels = new Collection<ZoomLevel>();
            resultZoomLevels.Add(zoomLevelSet.ZoomLevel01);
            resultZoomLevels.Add(zoomLevelSet.ZoomLevel02);
            resultZoomLevels.Add(zoomLevelSet.ZoomLevel03);
            resultZoomLevels.Add(zoomLevelSet.ZoomLevel04);
            resultZoomLevels.Add(zoomLevelSet.ZoomLevel05);
            return resultZoomLevels;
        }
    }
}
